local bn_assets = require("BattleNetwork.Assets")
local TEXTURE = bn_assets.load_texture("bn3_lightning.png")
local ANIM_PATH = bn_assets.fetch_animation_path("bn3_lightning.animation")
local FLASH_TEXTURE = bn_assets.load_texture("flashlight.png")
local FLASH_ANIM_PATH = bn_assets.fetch_animation_path("flashlight.animation")
local AUDIO = bn_assets.load_audio("dollthunder.ogg")

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_HIT")
	local long_frame = { { 2, 19 }, { 2, 26 } }
	action:override_animation_frames(long_frame)

	action:set_lockout(ActionLockout.new_animation())

	action.on_execute_func = function(self, user)
		local flash = Artifact.new()
		flash:set_texture(FLASH_TEXTURE)
		flash:animation():load(FLASH_ANIM_PATH)
		flash:animation():set_state("DEFAULT")
		flash:animation():on_complete(function()
			flash:erase()
		end)

		local field = user:field()

		field:spawn(flash, field:tile_at(1, 1))

		local targets = field:find_obstacles(function(found)
			if not found:hittable() then return false end
			if found:team() == user:team() then return false end
			return true
		end)

		self:add_anim_action(2, function()
			local tile = nil
			local spell = nil

			for i = 1, #targets, 1 do
				tile = targets[i]:current_tile()
				spell = create_bolt(user, props)
				field:spawn(spell, tile)
			end
		end)
	end
	return action
end

function create_bolt(user, props)
	local spell = Spell.new(user:team())

	spell:set_facing(user:facing())
	spell:set_tile_highlight(Highlight.Solid)
	spell:set_hit_props(
		HitProps.new(
			props.damage,
			props.hit_flags,
			props.element,
			user:context(),
			Drag.None
		)
	)
	local anim = spell:animation()
	spell:set_texture(TEXTURE)

	anim:load(ANIM_PATH)
	anim:set_state("DEFAULT")

	anim:apply(spell:sprite())
	anim:on_complete(function()
		spell:erase()
	end)

	spell.on_update_func = function(self)
		local tile = self:current_tile()
		self:attack_tiles(
			{
				tile,
				tile:get_tile(Direction.Up, 1),
				tile:get_tile(Direction.Down, 1),
				tile:get_tile(Direction.Left, 1),
				tile:get_tile(Direction.Right, 1)
			}
		)
	end

	spell.can_move_to_func = function(tile)
		return true
	end

	Resources.play_audio(AUDIO)

	return spell
end
